@@ -1,5 +1,5 @@
 ;;; gnus-start.el --- startup functions for Gnus
-;; Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001
+;; Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002
 ;;        Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
@@ -409,11 +409,8 @@ Can be used to turn version control on or off."
 
 ;;; Internal variables
 
-(defvar gnus-startup-file-coding-system 'binary
-  "*Coding system for startup file.")
-
 (defvar gnus-ding-file-coding-system mm-universal-coding-system
-  "*Coding system for ding file.")
+  "Coding system for ding file.")
 
 (defvar gnus-newsrc-file-version nil)
 (defvar gnus-override-subscribe-method nil)
@@ -441,25 +438,15 @@ Can be used to turn version control on or off."
     (if gnus-init-inhibit
 	(setq gnus-init-inhibit nil)
       (setq gnus-init-inhibit inhibit-next)
-      (let ((files (list gnus-site-init-file gnus-init-file))
-	    file)
-	(while files
-	  (and (setq file (pop files))
-	       (or (and (file-exists-p file)
-			;; Don't try to load a directory.
-			(not (file-directory-p file)))
-		   (file-exists-p (concat file ".el"))
-		   (file-exists-p (concat file ".elc")))
-	       (if (or debug-on-error debug-on-quit)
-		   (let ((coding-system-for-read
-			  gnus-startup-file-coding-system))
-		     (load file nil t))
-		 (condition-case var
-		     (let ((coding-system-for-read
-			    gnus-startup-file-coding-system))
-		       (load file nil t))
-		   (error
-		    (error "Error in %s: %s" file var))))))))))
+      (dolist (file (list gnus-site-init-file gnus-init-file))
+	(when (and file
+		   (locate-library file))
+	  (if (or debug-on-error debug-on-quit)
+	      (load file nil t)
+	    (condition-case var
+		(load file nil t)
+	      (error
+	       (error "Error in %s: %s" file var)))))))))
 
 ;; For subscribing new newsgroup
 
@@ -624,7 +611,7 @@ the first newsgroup."
 (defun gnus-clear-system ()
   "Clear all variables and buffers."
   ;; Clear Gnus variables.
-  (let ((variables gnus-variable-list))
+  (let ((variables (delete 'gnus-format-specs gnus-variable-list)))
     (while variables
       (set (car variables) nil)
       (setq variables (cdr variables))))
@@ -1603,7 +1590,7 @@ newsgroup."
 	  (when (and (<= (gnus-info-level info) foreign-level)
 		     (setq active (gnus-activate-group group 'scan)))
 	    ;; Let the Gnus agent save the active file.
-	    (when (and gnus-agent gnus-plugged active)
+	    (when (and gnus-agent active (gnus-online method))
 	      (gnus-agent-save-group-info
 	       method (gnus-group-real-name group) active))
 	    (unless (inline (gnus-virtual-group-p group))
@@ -1910,7 +1897,7 @@ newsgroup."
 	(insert ?\\)))
 
     ;; Let the Gnus agent save the active file.
-    (when (and gnus-agent real-active gnus-plugged)
+    (when (and gnus-agent real-active (gnus-online method))
       (gnus-agent-save-active method))
 
     ;; If these are groups from a foreign select method, we insert the
@@ -1986,7 +1973,7 @@ newsgroup."
     ;; Let the Gnus agent save the active file.
     (if (and gnus-agent
 	     real-active
-	     gnus-plugged
+	     (gnus-online method)
 	     (gnus-agent-method-p method))
 	(progn
 	  (gnus-agent-save-groups method)
@@ -2027,7 +2014,7 @@ newsgroup."
   "Read startup file.
 If FORCE is non-nil, the .newsrc file is read."
   ;; Reset variables that might be defined in the .newsrc.eld file.
-  (let ((variables gnus-variable-list))
+  (let ((variables (delete 'gnus-format-specs gnus-variable-list)))
     (while variables
       (set (car variables) nil)
       (setq variables (cdr variables))))
@@ -2112,8 +2099,8 @@ If FORCE is non-nil, the .newsrc file is read."
 	     (and gnus-newsrc-file-version
 		  (gnus-continuum-version gnus-newsrc-file-version))))
 	(when (or (not version)
-		  (< version 5.090002))
-	  (setq gnus-format-specs nil)))
+		  (< version 5.090009))
+	  (setq gnus-format-specs gnus-default-format-specs)))
       (when gnus-newsrc-assoc
 	(setq gnus-newsrc-alist gnus-newsrc-assoc)))
     (gnus-make-hashtable-from-newsrc-alist)
